/*
  Simple Demo for GLSL
  www.lighthouse3d.com
  
  tweaked a bit by andy
*/

#include "GL/glew.h"

#ifdef __linux__
#include <GL/glut.h>
#endif

#ifdef __APPLE__
#include <glut.h>
#endif

#ifdef _WIN32
#include "glut.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>



char greenData[800000];
char blueData[800000];
char purpleData[800000];
char yellowData[800000];
char whiteData[800000];
char redData[800000];

char backData[800000];

GLuint redtex, greentex, bluetex, yellowtex, whitetex, purpletex, backtex;
GLuint texNumNormal;

float mouseWindowX = 0.0;
float mouseWindowY = 0.0;

int windowWidth = 320;
int windowHeight = 320;

int whichToShow = 0;

GLuint texNumBack, texNumCube;

////////////////////////////////////////////////////////////////////////////////

char *textFileRead(char *fn)
{
	FILE *fp;
	char *content = NULL;

	int count=0;

	if (fn != NULL) {
		fp = fopen(fn,"rt");

		if (fp != NULL) {
      
      fseek(fp, 0, SEEK_END);
      count = ftell(fp);
      rewind(fp);

			if (count > 0) {
				content = (char *)malloc(sizeof(char) * (count+1));
				count = fread(content,sizeof(char),count,fp);
				content[count] = '\0';
			}
			fclose(fp);
		}
	}
	
	if (content == NULL)
	   {
	   fprintf(stderr, "ERROR: could not load in file %s\n", fn);
	   exit(1);
	   }
	return content;
}            


////////////////////////////////////////////////////////////////////////////////

GLuint p;

float lpos[4] = {1.0, 0.5, 1.0, 0.0};

void changeSize(int w, int h) {

	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;

	float ratio = 1.0* w / h;

	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	// Set the viewport to be the entire window
    glViewport(0, 0, w, h);

    windowWidth = w;
    windowHeight = h;
    
	// Set the correct perspective.
	gluPerspective(45, ratio, 1, 1000);
	glMatrixMode(GL_MODELVIEW);
}

////////////////////////////////////////////////////////////////////////////////

void printShaderLog(GLuint prog)
{
    GLint infoLogLength = 0;
    GLsizei charsWritten  = 0;
    GLchar *infoLog;

    glGetShaderiv(prog, GL_INFO_LOG_LENGTH, &infoLogLength);

    if (infoLogLength > 0)
    {
        infoLog = (char *) malloc(infoLogLength);
        glGetShaderInfoLog(prog, infoLogLength, &charsWritten, infoLog);
		printf("%s\n",infoLog);
        free(infoLog);
    }
}

void printProgramLog(GLuint shad)
{
    GLint infoLogLength = 0;
    GLsizei charsWritten  = 0;
    GLchar *infoLog;

    glGetProgramiv(shad, GL_INFO_LOG_LENGTH, &infoLogLength);

    if (infoLogLength > 0)
    {
        infoLog = (char *) malloc(infoLogLength);
        glGetProgramInfoLog(shad, infoLogLength, &charsWritten, infoLog);
		printf("%s\n",infoLog);
        free(infoLog);
    }
}

////////////////////////////////////////////////////////////////////////////////

float rotAmount = 0;

void renderScene(void) {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	gluLookAt(0.0,0.0,3.0, 
		      0.0,0.0,-1.0,
			  0.0f,1.0f,0.0f);

	glLightfv(GL_LIGHT0, GL_POSITION, lpos);
	glRotatef(rotAmount,0,1,0);

	// turn off any shaders for now
	glUseProgram(0);

    glEnable(GL_TEXTURE_2D); 
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, texNumBack);

	// draw a square polygon with the background texture on it
    glColor3f(1.0, 1.0, 1.0);
    glBegin( GL_POLYGON );
        glTexCoord2f( 0.0f, 1.0f );
        glVertex3f( -2.5f, -2.5f, -2.0f );
        
        glTexCoord2f( 1.0f, 1.0f ); 
        glVertex3f(  2.5f, -2.5f, -2.0f );
        
        glTexCoord2f( 1.0f, 0.0f );
        glVertex3f(  2.5f,  2.5f, -2.0f );
        
        glTexCoord2f( 0.0f, 0.0f ); 
        glVertex3f( -2.5f,  2.5f, -2.0f );
   glEnd();




	glRotatef(mouseWindowX*90+90,0,1,0);
	glRotatef(mouseWindowY*90,0,0,1);

	// turn on the shaders
    glUseProgram(p);

    GLint texLoc, lightLoc, whichLoc;

    whichLoc = glGetUniformLocation(p, "which");
    glUniform1i(whichLoc, whichToShow);

    lightLoc = glGetUniformLocation(p, "LightPos");
    glUniform3f(lightLoc, 1.0, 1.0, 4.0);
    
    texLoc = glGetUniformLocation(p, "envMap");
    glUniform1i(texLoc, 0);
    
    glutSolidTeapot(1);

   // rotAmount+= 0.1;

	glutSwapBuffers();
}

////////////////////////////////////////////////////////////////////////////////

void processNormalKeys(unsigned char key, int x, int y) {

	if (key == 27) 
		exit(0);

     if (key == '1') 
		{
		whichToShow = 0;
		}
    if (key == '2') 
		{
		whichToShow = 1;
		}
    if (key == '3') 
		{
		whichToShow = 2;
		}
}


////////////////////////////////////////////////////////////////////////////////

GLuint setShaders(char * vert, char * frag) {
    GLuint v,f, pro;
	char *vs,*fs;

	v = glCreateShader(GL_VERTEX_SHADER);
	f = glCreateShader(GL_FRAGMENT_SHADER);

	vs = textFileRead(vert);
	fs = textFileRead(frag);

	const char * vv = vs;
	const char * ff = fs;

	glShaderSource(v, 1, &vv,NULL);
	glShaderSource(f, 1, &ff,NULL);

	free(vs);free(fs);

	glCompileShader(v);
	glCompileShader(f);

	printShaderLog(v);
	printShaderLog(f);

	pro = glCreateProgram();
	glAttachShader(pro,v);
	glAttachShader(pro,f);

	glLinkProgram(pro);
	printProgramLog(pro);
	
	return(pro);
}

////////////////////////////////////////////////////////////////////////////////

GLuint setupTexture(char * texData)
{
	GLuint textureNum[1];

	glGenTextures(1, textureNum);

    glBindTexture(GL_TEXTURE_2D, textureNum[0]);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 512, 512, 0, GL_RGB,
        GL_UNSIGNED_BYTE, (const GLvoid *) texData);

    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

    return(textureNum[0]);
}

////////////////////////////////////////////////////////////////////////////////

void doTexStuff(char * dataPath, char * fileName, int width, int height,
		char * TheArray)
    {
    register int x, y;
    char fullPath[256];
    int arrayLoc;
    FILE * imageFile;
    int maxVal =0;
    float ratio;

    strcpy(fullPath, dataPath);
    strcat(fullPath, fileName);
    imageFile = fopen(fullPath, "r");
	
    if (imageFile == NULL) 
	   fprintf(stderr,"Cannot find texture file in data directory");
    else
        {	    
        for (y = 0; y < height*width*3; y+=1) /// rgb texture this time ...
           {
            TheArray[y]	= (char) fgetc(imageFile);
            }
                    
        fclose(imageFile);
        }

    }

////////////////////////////////////////////////////////////////////////////////

void PassiveMouseMotion(int x, int y)
{
    if (x<0)
        mouseWindowX = -1;
    else if (x>windowWidth)
        mouseWindowX = 1;
    else
        mouseWindowX = 2* (x / (float) windowWidth) - 1.0;
           
    if (y<0)
        mouseWindowY = -1;
    else if (y>windowHeight)
        mouseWindowY = 1;
    else
        mouseWindowY = 2* (y / (float) windowHeight) - 1.0;
    }

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char **argv) {
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(windowWidth, windowHeight);
	glutCreateWindow("GLSL");

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);
	glutReshapeFunc(changeSize);
	glutKeyboardFunc(processNormalKeys);
	glutPassiveMotionFunc(PassiveMouseMotion);

	glewInit();
	
	glEnable(GL_DEPTH_TEST);
	glClearColor(0.0,0.0,0.0,1.0);

    doTexStuff((char *) "./", (char *) "white.raw",  512, 512, whiteData);
    doTexStuff((char *) "./", (char *) "red.raw",    512, 512, redData);
    doTexStuff((char *) "./", (char *) "blue.raw",   512, 512, blueData);
    doTexStuff((char *) "./", (char *) "green.raw",  512, 512, greenData);
    doTexStuff((char *) "./", (char *) "sq.raw",     512, 512, purpleData);
    doTexStuff((char *) "./", (char *) "yellow.raw", 512, 512, yellowData);
 
    // set up a texture to appear in the background
    texNumBack = setupTexture(purpleData);

    // now set up the cube map
    GLuint textureNum[1];
	glGenTextures(1, textureNum);
	texNumCube = textureNum[0];
    glBindTexture(GL_TEXTURE_CUBE_MAP, texNumCube);
     
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_REPEAT);
    
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    
    glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X, 
        0, GL_RGB, 512, 512,  0, GL_RGB, GL_UNSIGNED_BYTE, &redData);
    glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X, 
        0, GL_RGB, 512, 512,  0, GL_RGB, GL_UNSIGNED_BYTE, &blueData);
        
    glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y, 
        0, GL_RGB, 512, 512, 0, GL_RGB, GL_UNSIGNED_BYTE, &greenData);
    glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y, 
        0, GL_RGB, 512, 512,  0, GL_RGB, GL_UNSIGNED_BYTE, &yellowData);   
        
    glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z, 
        0, GL_RGB, 512, 512,  0, GL_RGB, GL_UNSIGNED_BYTE, &whiteData);
    glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z, 
        0, GL_RGB, 512, 512, 0, GL_RGB, GL_UNSIGNED_BYTE, &purpleData);



	glewInit();
	if (GLEW_ARB_vertex_shader && GLEW_ARB_fragment_shader)
		printf("Ready for GLSL\n");
	else {
		printf("No GLSL support\n");
		exit(1);
	}

	p = setShaders((char *) "./refract.vert", (char *) "./refract.frag");
	
	glutMainLoop();
	return 0;
}

