// from the Orange Book 2nd edition

const float Eta = 0.66;

const float EtaR = 0.65;
const float EtaG = 0.67;
const float EtaB = 0.69;

const float FresnelPower = 0.5;
const float F = ((1.0-Eta)*(1.0-Eta))/((1.0+Eta)*(1.0+Eta));

const float FR = ((1.0-EtaR)*(1.0-EtaR))/((1.0+EtaR)*(1.0+EtaR));
const float FG = ((1.0-EtaG)*(1.0-EtaG))/((1.0+EtaG)*(1.0+EtaG));
const float FB = ((1.0-EtaB)*(1.0-EtaB))/((1.0+EtaB)*(1.0+EtaB));


varying vec3 Reflect;
varying vec3 Refract;

varying vec3 RefractR;
varying vec3 RefractG;
varying vec3 RefractB;

varying float Ratio;

varying float RatioR;
varying float RatioG;
varying float RatioB;

void main()
{
vec4 ecPosition = gl_ModelViewMatrix * gl_Vertex;
vec3 ecPosition3 = ecPosition.xyz / ecPosition.w;

vec3 i = normalize(ecPosition3);
vec3 n = normalize(gl_NormalMatrix * gl_Normal);

Ratio = F + (1.0 - F) * pow((1.0 - dot(-i, n)), FresnelPower);

RatioR = FR + (1.0 - FR) * pow((1.0 - dot(-i, n)), FresnelPower);
RatioG = FG + (1.0 - FG) * pow((1.0 - dot(-i, n)), FresnelPower);
RatioB = FB + (1.0 - FB) * pow((1.0 - dot(-i, n)), FresnelPower);

Refract = refract(i, n, Eta);
Refract = vec3(gl_TextureMatrix[0] * vec4 (Refract, 1.0));

RefractR = refract(i, n, EtaR);
RefractR = vec3(gl_TextureMatrix[0] * vec4 (RefractR, 1.0));
RefractG = refract(i, n, EtaG);
RefractG = vec3(gl_TextureMatrix[0] * vec4 (RefractG, 1.0));
RefractB = refract(i, n, EtaB);
RefractB = vec3(gl_TextureMatrix[0] * vec4 (RefractB, 1.0));

Reflect = reflect(i, n);
Reflect = vec3(gl_TextureMatrix[0] * vec4 (Reflect, 1.0));

gl_Position = ftransform();
}